import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


public class ControlCanvas extends ScaleCanvas implements MouseListener,MouseMotionListener {
    Manager M;
    PolygonWrapper[] POLY=new PolygonWrapper[2];
    double[][] SEGMENT=new double[50][5];
    int COUNT=0;
    Color[] COL=new Color[2];
 

     public ControlCanvas() {
	 addMouseListener(this);
	 addMouseMotionListener(this);	
	 setScales(0,400,400);
	 setPolygon();
     }


    public void setPolygon() {
	Complex z0=new Complex(.1,.1);
	Complex z1=new Complex(.9,.9);
	Complex z2=new Complex(.9,.1);
	Complex[] Z={z2,z1,z0};
	POLY[1]=new PolygonWrapper(3,Z);

	Complex w0=new Complex(.8,.8);
	Complex w1=new Complex(.2,.9);
	Complex w2=new Complex(.5,.5);
	Complex[] W={w0,w1,w2};
	POLY[0]=new PolygonWrapper(3,W);

	COL[0]=Color.blue;
	COL[1]=new Color(200,0,0);
    }

   public void paint(Graphics g2) {
      Graphics2D g=(Graphics2D) g2;
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      drawBG(g);
      drawPolys(g);
      getSegments();
      drawGeodesic(g);
      drawCross(g,M.P.MARKER);
   }

    public void drawBG(Graphics2D g) {
      g.setColor(new Color(0,0,100));
      g.fillRect(0,0,getWidth(),getHeight()); 
      Path2D.Double gp=new Path2D.Double();
      gp.moveTo(0,0);
      gp.lineTo(1,0);
      gp.lineTo(1,1);
      gp.lineTo(0,1);
      gp.closePath();
      gp=transform(gp);
      g.setColor(new Color(255,120,0));
      g.fill(gp);
    }


    public void drawPolys(Graphics2D g) {
	for(int i=0;i<2;++i) {
	  Path2D.Double gp=POLY[i].toGeneralPath(); 
          g.setColor(COL[i]);
          gp=transform(gp);
          g.fill(gp);
	}
    }

    public void drawGeodesic(Graphics2D g) {
	g.setStroke(new BasicStroke(2));
	Path2D.Double gp=new Path2D.Double();
	for(int i=0;i<COUNT;++i) {
	    gp=segmentPath(SEGMENT[i]);
	    gp=transform(gp);
	    g.setColor(Color.white);
	    g.draw(gp); 
	}
	g.setStroke(new BasicStroke(1));
    }




    public void drawCross(Graphics2D g,double x) {
	Path2D.Double gp=new Path2D.Double();
	double[] X=getPoint(x);
	Complex z=new Complex(X[0],X[1]);
	drawPoint(g,z,.015,Color.yellow);
    }

    public void drawPoint(Graphics2D g,Complex z,double r,Color C) {
	Path2D.Double gp=new Path2D.Double();
	for(int i=0;i<64;++i) {
	    double t=2.0*i*Math.PI/64;
	    double c=z.x+r*Math.cos(t);
	    double s=z.y+r*Math.sin(t);
	    if(i==0) gp.moveTo(c,s);
	    if(i!=0) gp.lineTo(c,s);
	}
	g.setColor(C);
	gp=transform(gp);
	g.fill(gp);
	g.draw(gp);
    }

    public static double[] basicMap(double s,double[] t) {
	if(t[0]<.5) {
	    double[] u={1,s*(1-t[1])};
	    return u;
	}
	if(t[1]<1-s) {
	    double[] u={1,t[1]+s};
	    return u;
	}
	double[] u={0,(1-t[1])/s};
	return u;
    }


    public static double[] endpoint0(double[] t) {
	if(t[0]<.5) {
	    double[] u={t[1],0};
	    return u;
	}
	double[] u={0,t[1]};
	return u;
    }


    public static double[] endpoint1(double[] t) {
	if(t[0]<.5) {
	    double[] u={t[1],1};
	    return u;
	}
	double[] u={1,t[1]};
	return u;
    }


    public static Path2D.Double segmentPath(double[] X) {
	Path2D.Double gp=new Path2D.Double();
	gp.moveTo(X[0],X[1]);
	gp.lineTo(X[2],X[3]);
	gp.closePath();
	return gp;
    }

    public void getSegments() {
	int count=0;
	double s=M.P.SLOPE.getValue();
	double[] t={0,0};
	double total=0;
	Path2D.Double gp=new Path2D.Double();
	while(total<4) {
	    SEGMENT[count]=segment(s,t);
	    total=total+SEGMENT[count][4];
	    ++count;
            t=basicMap(s,t);
	}
	COUNT=count;
	int c=COUNT-1;
	double excess=total-4;
	double L=SEGMENT[c][4];
	double f=(L-excess)/L;
	SEGMENT[c][2]=(1-f)*SEGMENT[c][0]+f*SEGMENT[c][2];
	SEGMENT[c][3]=(1-f)*SEGMENT[c][1]+f*SEGMENT[c][3];
	SEGMENT[c][4]=L-excess;
    }



    public static double[] segment(double s,double[] t) {
	double[] u=basicMap(s,t);
	double[] tt=endpoint0(t);
	double[] uu=endpoint1(u);
	double d0=(tt[0]-uu[0])*(tt[0]-uu[0]);
	double d1=(tt[1]-uu[1])*(tt[1]-uu[1]);
	double d=Math.sqrt(d0+d1);
	double[] X={tt[0],tt[1],uu[0],uu[1],d};
	return X;
    }


    public double[] getPoint(double s) {
	if(COUNT==0) {
	    double[] X={0,0};
	    return X;
	}

	double total=0;
	int count=0;

	while(total<s) {
	    total=total+SEGMENT[count][4];
	    ++count;
	}
	int c=count-1;
	double excess=total-s;
	double L=SEGMENT[c][4];
	double f=(L-excess)/L;
	double x=(1-f)*SEGMENT[c][0]+f*SEGMENT[c][2];
	double y=(1-f)*SEGMENT[c][1]+f*SEGMENT[c][3];
	double[] X={x,y};
	return X;
    }



    public void mouseClicked(MouseEvent e) {
        MouseData J=MouseData.process(e);
	repaint();
	M.P.repaint();
    }

    public void mousePressed(MouseEvent e) {
        MouseData J=MouseData.process(e);
	repaint();
    }

     public void mouseReleased(MouseEvent e) {}
     public void mouseEntered(MouseEvent e) {}
     public void mouseExited(MouseEvent e) {}   
     public void mouseMoved(MouseEvent e) {
     }   

     public void mouseDragged(MouseEvent e) {
        MouseData J=MouseData.process(e);
	repaint();
	M.P.repaint();
     } 


}